/*
  * Setup the message queue and check the queue in the run function,
  * which is called from a QTimer. 
  * Handle the messages received from the queue. 
  * 
  * Author: John Davin
  */

#ifndef CHECK_QUEUE_H
#define CHECK_QUEUE_H

#include <qtimer.h>
#include ".ui/actionsDisplay.h"


/* The following is a hack to get g++ to allow us to redefine struct msgbuf (in ActionsGUI.C) 
 * over the definition in sys/msg.h. 
 *
 */
#ifdef __USE_GNU
#undef __USE_GNU
#include <sys/msg.h>
#define __USE_GNU
#else
#include <sys/msg.h>
#endif

#define MSGSIZE 128
typedef struct msgbuf {
    long    mtype;
    char    mtext[MSGSIZE];
} message_buf;


#define NUM_PLAYERS 11
// color to gray out cells if not updated in past 1 second
#define GRAYED_OUT "gray80"


class CheckQueue : public QObject {
    Q_OBJECT
    
    public:
    CheckQueue(ActionsDisplay *ptr, QTimer *timer);
    ~CheckQueue();

    void init();

    public slots:

    virtual void run();
    

    private:
    int MsqID;
    ActionsDisplay *actionsdisp;
    int lastreceived[NUM_PLAYERS];
    QTimer *Timer;
    int idleInterval; // time interval to run the run() function
    
    void handleMsg(char *msg);
    void applyStyle(int row, const char *actionsrc, const char *msg, char *fgcolor, char *bgcolor);

};


#endif

