/* Dongryeol Lee
 *
 * ActionPicker.h
 * 
 * This class accumulates the action sent into the storage, and
 * given the right moment, picks an action and returns it.
 */

#include <vector>
#include "ActionBinding.h"
#include "clangaction.h"
#include "VarBindings.h"
#include "SoccerTypes.h"


#ifndef ACTIONPICKER
#define ACTIONPICKER


class ActionPicker
{
 private:

  /* Private member variables */

  /* Store our list of default actions here: this list never gets cleared */
  std::vector <ActionBinding> defaultActions;


  /* Store our list of actions here: this list gets cleared every cycle
     every time the agent makes a decision */
  std::vector <ActionBinding> listOfActions;

  /* Store the ID of the previously executed home advice action */
  Action::ActionID prevHomeID;
  /* the last time that we set the homeID */
  Time time_last_home_set;


  /* Private member functions */

  /* Reduce and filter our list of advice */
  const std::vector <ActionBinding> reduceAndFilter();

  /* Consider our advice set */
  bool considerAdviceSet(const int &level);

  /* Consider our default advice set */
  bool considerDefaultAdvice(const int& level) const;

  void addCommunicationActions();

  void cleanup();


 public:

  /* Constructor */
  ActionPicker();

  /* Add action to the Action Picker */
  void addAction(rcss::clang::Action *addThisAction, 
		 const VarBindings &bindings, 
		 ActionBinding::SourceType source);

  /* Execute an action from the Action Picker */
  void pickAction();

  /* Clear our action list */
  void clear();

};
  
#endif
