/* This is the ActionVisitor Class, which extracts
   information from a given action */

#include "visitor.h"
#include "clangutil.h"
#include "clangaction.h"
#include "vector.h"
#include "SoccerTypes.h"
#include "Geometry.h"
#include "ActionVisitor.h"
#include "msgstore.h"
#include <stdio.h>
#include <typeinfo>
#include "config.h"

using namespace rcss::clang;

extern MsgStore <Action *> defactstorage;


const Region * ActionVisitor::getRegion() const
{
  return reg;
}

const ActionVisitor::ActionType ActionVisitor::getActionType() const
{
  return actionType;
}

const UNumSet& ActionVisitor::getPlayers() const
{
  return players;
}

const BallMove ActionVisitor::getBallMove() const
{
  return ballMove;
}

const int ActionVisitor::getPlayerType() const
{
  return playerType;
}

/* Visit ActPos action */
void ActionVisitor::visit(ActPos *act)
{
  actionType = POS;  /* record action type */
  reg = (Region *) act->getRegion(); /*extract region */
}

/* Visit ActHome action */
void ActionVisitor::visit(ActHome *act)
{
  actionType = HOME; /* Record action type */
  reg = (Region *) act->getRegion();   /* extract region */
}

/* Visit ActBallToReg action */
void ActionVisitor::visit(ActBallToReg *act)
{
  actionType = BALLTOREG; /* Record action type */
  reg = (Region *) act->getRegion();  /* extract region */
  ballMove = act->getBallMove();  /* get ball information */
}

/* Visit ActBallToPlayer action */
void ActionVisitor::visit(ActBallToPlayer *act)
{
  actionType = BALLTOPLAYER; /* record action type */
  players = act->getPlayers(); /* retrieve players */
}

/* Visit ActMark action */
void ActionVisitor::visit(ActMark *act)
{
  actionType = MARK; /* record action type */
  players = act->getPlayers(); /* retrieve players */
}

/* Visit ActMarkLinePlayer action */
void ActionVisitor::visit(ActMarkLinePlayer *act)
{
  actionType = MARKLINEPLAYER; /* record action type */
  players = act->getPlayers(); /* retrieve players */
}

/* visit ActMarkLineReg action */
void ActionVisitor::visit(ActMarkLineReg *act)
{
  actionType = MARKLINEREG; /* record action type */
  reg = (Region *) act->getRegion();   /* retrieve region */
}

/* visit ActOffSidesLine action */
void ActionVisitor::visit(ActOffsidesLine *act)
{
  actionType = OFFSIDESLINE; /* record action type */
  reg = (Region *) act->getRegion();  /* retrieve region */
}

/* visit ActHetType action */
void ActionVisitor::visit(ActHetType *act)
{
  actionType = HETTYPE;
  playerType = act->getPlayerType();
}

/* visit ActNamed action */
void ActionVisitor::visit(ActNamed *act)
{
  ActionVisitor avisitor(myBindings);

  /* Get the action corresponding to the name from the storage
     and visit it */
  Action *a = (Action *) act->getAction();

  a->accept(*this);
}

/* visit ActPassReg action */
void ActionVisitor::visit(ActPassReg *act)
{
  actionType = PASSREG;
  reg = (Region *) act->getRegion();
}

/* visit ActPassUNum action */
void ActionVisitor::visit(ActPassUNum *act)
{
  actionType = PASSUNUM;
  players = act->getPlayers();
  UNumSet newset;

  /* iterate over the UNums in the players UNumSet to see if any are 
     variables:  */
  for(UNumSet::iterator i = players.begin(); i != players.end(); i++)  {
    
    if((*i).getVar() != ""){ // then we have named variable
      
      /*
      Log.log(10, 
      "[ActionVisitor]: Found variable in action. Expanding from bindings.\n");
      */
      
      UNumSet varset = myBindings.lookup((*i).getVar());

      /*
      if(varset.begin() == varset.end()) // empty set
	Log.log(9, 
		"[ActionVisitor]: ERROR: Variable %s is not bound!\n", 
		(*i).getVar().c_str());
      */
	
      // add the varset to the newset. 
      for(UNumSet::iterator u = varset.begin(); u != varset.end(); u++)  {
	
	newset.add(*u);
	
      }

      /*
      if(Log.isInLogLevel(10)) 
	*logger << 
	  "[ActionVisitor]: Expanded "<<(*i).getVar()<<" to "<<varset<<endl;
      */

    }
    else // just a regular UNum
      newset.add(*i);
        
  }

  players = newset; 
}

/* visit ActDribble action */
void ActionVisitor::visit(ActDribble *act)
{
  actionType = DRIBBLE;
  reg = (Region *) act->getRegion();
}

/* visit ActClear action */
void ActionVisitor::visit(ActClear *act)
{
  actionType = CLEAR;
  reg = (Region *) act->getRegion();
}

/* visit ActShoot action */
void ActionVisitor::visit(ActShoot *)
{
  actionType = SHOOT;
}

/* visit ActHold action */
void ActionVisitor::visit(ActHold *)
{
  actionType = HOLD;
}

/* visit ActIntercept action */
void ActionVisitor::visit(ActIntercept *)
{
  actionType = INTERCEPT;
}

/* visit ActTackle action */
void ActionVisitor::visit(ActTackle *act)
{
  actionType = TACKLE;
  players = act->getPlayers();
}

