#include "visitor.h"
#include "clangutil.h"
#include "clangaction.h"
#include "region.h"
#include "Geometry.h"
#include "WorldModel.h"
#include "VarBindings.h"

#ifndef ACTIONVISITOR
#define ACTIONVISITOR

using namespace rcss::clang;

/* This class is the action visitor class, which 
   extracts information about an action */
class ActionVisitor: public Action::Visitor
{
 public:
  enum ActionType {POS, HOME, BALLTOREG, BALLTOPLAYER, MARK, MARKLINEPLAYER,
		   MARKLINEREG, OFFSIDESLINE, HETTYPE, NAMED, PASSREG,
		   PASSUNUM, DRIBBLE, CLEAR, SHOOT, HOLD, INTERCEPT,
		   TACKLE};

  ActionVisitor() {}

  ActionVisitor(const VarBindings& b)
		: myBindings(b)
		{}
  
  const Region *getRegion() const;

  const ActionType getActionType() const;
  
  const UNumSet& getPlayers() const;

  const BallMove getBallMove() const;

  const int getPlayerType() const;

  void visit(ActPos *act);

  void visit(ActHome *act);
  
  void visit(ActBallToReg *act);
  
  void visit(ActBallToPlayer *act);
  
  void visit(ActMark *act);
  
  void visit(ActMarkLinePlayer *act);
  
  void visit(ActMarkLineReg *act);

  void visit(ActOffsidesLine *act);

  void visit(ActHetType *act);

  void visit(ActNamed *act);

  void visit(ActPassReg *act);

  void visit(ActPassUNum *act);

  void visit(ActDribble *act);

  void visit(ActClear *act);

  void visit(ActShoot *act);

  void visit(ActHold *act);

  void visit(ActIntercept *act);

  void visit(ActTackle *act);

 private:
  Region *reg;

  ActionType actionType;

  BallMove ballMove;

  UNumSet players;

  int playerType;

  VarBindings myBindings; // bindings for variable present in actions

};

#endif

  
