/* Dongryeol Lee
 * 
 * CompareActionVisitor.h
 *
 * This class is the visitor class that will compare two actions and store
 * the result as a boolean truth value.
 *
 */


#ifndef COMPARING_ACTION_VISITOR
#define COMPARING_ACTION_VISITOR


#include "ActionBinding.h"
#include "ActionVisitor.h"


class CompareActionVisitor: public Action::Visitor
{
 public:

  /* Constructor */
  CompareActionVisitor(const ActionBinding &abinding) { 
    
    /* Construct our action visitor */ 
    ActionVisitor compTo(abinding.myBindings);
    
    /* Call accept on the act */
    (abinding.theaction)->accept(compTo);

    /* Action type extracted */
    actionType = compTo.getActionType();

    /* Get players */
    players = compTo.getPlayers();
  }

  bool getResult() { return result; }

  void visit(ActPos *act);

  void visit(ActHome *act);
  
  void visit(ActBallToReg *act);
  
  void visit(ActBallToPlayer *act);
  
  void visit(ActMark *act);
  
  void visit(ActMarkLinePlayer *act);
  
  void visit(ActMarkLineReg *act);

  void visit(ActOffsidesLine *act);

  void visit(ActHetType *act);

  void visit(ActNamed *act);

  void visit(ActPassReg *act);

  void visit(ActPassUNum *act);

  void visit(ActDribble *act);

  void visit(ActClear *act);

  void visit(ActShoot *act);

  void visit(ActHold *act);

  void visit(ActIntercept *act);

  void visit(ActTackle *act);

 private:
  
  Region *reg;

  ActionVisitor::ActionType actionType;

  BallMove ballMove;

  UNumSet players;

  int playerType;

  VarBindings myBindings; // bindings for variable present in actions

  bool result;

};


#endif
