/* Dongryeol Lee
 *
 * Filter.h
 *
 * Header file for Filter class
 *
 * This class provides the filtering of advice using two filters:
 *
 * 1) Don't filter: When a directive is visited in DirVisitor, every "Don't"
 *                  directives will be added to the set here
 * 2) Applicability filter: Filter that examines the current world state and
 *                          determines whether the advice is aplicable
 *
 * In the preliminary stage of picking the best advice, the Action Picker will
 * use both of these filters to remove all invalid advice. An advice is said to
 * be valid if it passes both of these filters.
 *
 */


#include <vector>
#include "ActionBinding.h"
#include "ApplicabilityActionVisitor.h"


#ifndef FILTER
#define FILTER


using namespace rcss::clang;
using namespace std;


class Filter
{

 private:

  /* Store all of our "Don't" advice here */
  std::vector <ActionBinding> doNotList;
  
 public:

  /* Add "Don't" advice to the filter */
  void addDoNotAdvice(Action *addThisAction, const VarBindings &bindings, 
		      ActionBinding::SourceType source);

 public:
  
  /* Tells whether the given action binding should be filtered */
  bool shouldBeFiltered(const ActionBinding &abinding) const;

  void clear();


 private:

  /* Returns true if the advice is in the "Don't Do" list */
  bool isInDoNotList(const ActionBinding& abinding) const;

  /* Returns true if the advice fails the Applicability condition */
  bool failsApplicability(const ActionBinding &abinding) const;

};

#endif
