/* Dongryeol Lee
 *
 * LevelStorage.C
 *
 * Implementation of the storage class for Level objects
 *
 */


#include "LevelStorage.h"
#include "WorldModelAccessor.h" // for trilearn_wm


/* Intialize the storage with default sets of levels */
void LevelStorage::init()
{
  using namespace rcss::clang;
  

  /*  Action: Home; Source: Coach; Predicate: None */
  listOfLevels.push_back(Level(ActionVisitor::HOME,ActionBinding::COACH));
  
  /*  Action: Home; Source: Any; Predicate: None */
  listOfLevels.push_back(Level(ActionVisitor::HOME,ActionBinding::ANY));

  /*  Action: Shoot; Source: Coach; Predicate: None */
  listOfLevels.push_back(Level(ActionVisitor::SHOOT,ActionBinding::COACH));

  /*  Action: Pass; Source: Coach;
     Action Predicate: Knows where the passee is
                       Passee is in forward position than the agent 
		       It is a good, unobstructed pass (unobstructedness 
		       defined in the predicate itself)
  */
  listOfLevels.push_back(Level(ActionVisitor::PASSUNUM, 
			       ActionBinding::COACH).
			 addActionPredicate(new KnowsTeammatePos( true ) ).
			 addActionPredicate(new ForwardTeammate( true ) ).
			 addActionPredicate(new GoodPass( true ) ) );


  /**** COMMUNICATION Levels *****/
  /* Action: Pass; Source: Communication; Predicate: None
   */
  if (!trilearn_wm->getPlayerSettings()->getForbidNonCoachPass())
    listOfLevels.push_back(Level(ActionVisitor::PASSUNUM, 
				 ActionBinding::COMMUNICATION));

  /* Action: Intercept; Source: Communication; Predicate: None
   */
  listOfLevels.push_back(Level(ActionVisitor::INTERCEPT, 
			       ActionBinding::COMMUNICATION).
			 addActionPredicate(new IsBallOwner(false)).
			 addActionPredicate(new BallIsKickable(false)) );
  /*******************************/



  /*  Action: Dribble; Source: Coach;
     Action Predicate: Dribble region is uncongested ( < some threshold) */
  listOfLevels.push_back(Level(ActionVisitor::DRIBBLE,
			       ActionBinding::COACH).
			 addActionPredicate(new RegionUncongested(true) ));
  
  /*  Action: Pass; Source: Coach;
     Action Predicate: Knows where the passee is
                       Passee is in backward position than the agent 
		       It is a good, unobstructed pass (unobstructed-ness
		       defined in the predicate itself)
  */
  listOfLevels.push_back(Level(ActionVisitor::PASSUNUM,
			       ActionBinding::COACH).
			 addActionPredicate(new KnowsTeammatePos( true ) ).
			 addActionPredicate(new ForwardTeammate( false ) ).
			 addActionPredicate(new GoodPass( true ) ));

  /*  Action: Shoot; Source: Any; Predicate: None */
  listOfLevels.push_back(Level(ActionVisitor::SHOOT,ActionBinding::ANY));

  /*  Action: Shoot; Source: Default; Predicate: None */
  listOfLevels.push_back(Level(ActionVisitor::SHOOT,ActionBinding::DEFAULT));

  /*  Action: Clear; Source: Coach; Predicate: None */
  listOfLevels.push_back(Level(ActionVisitor::CLEAR,ActionBinding::COACH));

  /*  Action: Hold; Source: Coach; Predicate: None */
  listOfLevels.push_back(Level(ActionVisitor::HOLD,ActionBinding::COACH));
  
  /*  Action: Pass; Source: Coach;
     (Basically a derivative of Level 3, but lacks the GoodPass condition)
     
     Action Predicate: Knows where the passee is
                       Passee is in forward position than the agent 
  */
  listOfLevels.push_back(Level(ActionVisitor::PASSUNUM,
			       ActionBinding::COACH).
			 addActionPredicate(new KnowsTeammatePos( true ) ).
			 addActionPredicate(new ForwardTeammate( true ) ) );

  /* - Action: Dribble; Source: Coach;
     (Basically a derivative of Level 4, but lacks the RegionUncongested
     condition)

     Action Predicate: None
  */
  listOfLevels.push_back(Level(ActionVisitor::DRIBBLE,
			       ActionBinding::COACH));
  
  /* - Action: Pass; Source: Coach;
     (Basically a derivative of Level 5, but lacks the GoodPass condition)

     Action Predicate: Knows where the passee is
                       Passee is in backward position than the agent 
  */
  listOfLevels.push_back(Level(ActionVisitor::PASSUNUM,
			       ActionBinding::COACH).
			 addActionPredicate(new KnowsTeammatePos( true ) ).
			 addActionPredicate(new ForwardTeammate( false ) ));

  /* - Action: Pass; Source: Rulefile;
     Action Predicate: Knows where the passee is
                       Passee is in forward position than the agent 
		       It is a good, unobstructed pass (unobstructedness 
		       defined in the predicate itself)
  */
  if (!trilearn_wm->getPlayerSettings()->getForbidNonCoachPass() &&
      !trilearn_wm->getPlayerSettings()->getForbidMostNonCoachActs())
    listOfLevels.push_back(Level(ActionVisitor::PASSUNUM, 
				 ActionBinding::RULEFILE).
			   addActionPredicate(new KnowsTeammatePos( true ) ).
			   addActionPredicate(new ForwardTeammate( true ) ).
			   addActionPredicate(new GoodPass( true ) ) );
  
  /* - Action: Dribble; Source: Rulefile;
     Action Predicate: Dribble region is uncongested ( < some threshold) */
  if (!trilearn_wm->getPlayerSettings()->getForbidMostNonCoachActs())
    listOfLevels.push_back(Level(ActionVisitor::DRIBBLE,
                                 ActionBinding::RULEFILE).
                           addActionPredicate(new RegionUncongested(true) ));
  
  /* - Action: Pass; Source: Rulefile;
     Action Predicate: Knows where the passee is
                       Passee is in backward position than the agent 
		       It is a good, unobstructed pass (unobstructed-ness
		       defined in the predicate itself)
  */
  if (!trilearn_wm->getPlayerSettings()->getForbidNonCoachPass() &&
      !trilearn_wm->getPlayerSettings()->getForbidMostNonCoachActs())
    listOfLevels.push_back(Level(ActionVisitor::PASSUNUM,
				 ActionBinding::RULEFILE).
			   addActionPredicate(new KnowsTeammatePos( true ) ).
			   addActionPredicate(new ForwardTeammate( false ) ).
			   addActionPredicate(new GoodPass( true ) ));

  /* - Action: Clear; Source: Rulefile; Predicate: None */
  if (!trilearn_wm->getPlayerSettings()->getForbidMostNonCoachActs())
    listOfLevels.push_back(Level(ActionVisitor::CLEAR,ActionBinding::RULEFILE));

  /* - Action: Hold; Source: Rulefile; Predicate: None */
  listOfLevels.push_back(Level(ActionVisitor::HOLD,ActionBinding::RULEFILE));
  
  /* - Action: Pass; Source: Rulefile;
     (Basically a derivative of Level 3, but lacks the GoodPass condition)
     
     Action Predicate: Knows where the passee is
                       Passee is in forward position than the agent 
		       Not a close pass
  */
  if (!trilearn_wm->getPlayerSettings()->getForbidNonCoachPass() &&
      !trilearn_wm->getPlayerSettings()->getForbidMostNonCoachActs())
    listOfLevels.push_back(Level(ActionVisitor::PASSUNUM,
				 ActionBinding::RULEFILE).
			   addActionPredicate(new KnowsTeammatePos( true ) ).
			   addActionPredicate(new ForwardTeammate( true ) ).
			   addActionPredicate(new NotClosePass(true) ) );

  /* - Action: Dribble; Source: Rulefile;
     (Basically a derivative of Level 4, but lacks the RegionUncongested
     condition)

     Action Predicate: None
  */
  if (!trilearn_wm->getPlayerSettings()->getForbidMostNonCoachActs())
    listOfLevels.push_back(Level(ActionVisitor::DRIBBLE,
                                 ActionBinding::RULEFILE));
  
  /* - Action: Pass; Source: Rulefile;
     (Basically a derivative of Level 5, but lacks the GoodPass condition)

     Action Predicate: Knows where the passee is
                       Passee is in backward position than the agent
		       Not a close pass
  */
  if (!trilearn_wm->getPlayerSettings()->getForbidNonCoachPass() &&
      !trilearn_wm->getPlayerSettings()->getForbidMostNonCoachActs())
    listOfLevels.push_back(Level(ActionVisitor::PASSUNUM,
				 ActionBinding::RULEFILE).
			   addActionPredicate(new KnowsTeammatePos( true ) ).
			   addActionPredicate(new ForwardTeammate( false ) ).
			   addActionPredicate(new NotClosePass(true)) );


  /* - Action: Tackle; Source: Coach;
     Action Predicate: Opponent's distance to the ball is less than 5 m
  */
  listOfLevels.push_back(Level(ActionVisitor::TACKLE,ActionBinding::COACH).
  			 addActionPredicate(new OpponentCloseToBall
					    <ActTackle> ( true )));


  /* - Action: Dribble; Source: Default; Action Predicate: None */
  if (!trilearn_wm->getPlayerSettings()->getForbidMostNonCoachActs())
    listOfLevels.push_back(Level(ActionVisitor::DRIBBLE,ActionBinding::DEFAULT).
                           addWorldStatePredicate(new IsPlayOnMode( true ) ).
                           addWorldStatePredicate(new IsNearBall( true ) ) );
  

  /* - Action: Pos; Source: Coach;
     World State Predicate: The world is not on play-on mode */
  listOfLevels.push_back(Level(ActionVisitor::POS,ActionBinding::COACH).
			 addWorldStatePredicate(new IsPlayOnMode( false ) ) );
  

  /* - Action: Intercept; Source: Default;

     World State Predicate: Agent is near ball
  */
  listOfLevels.push_back(Level(ActionVisitor::INTERCEPT,
			       ActionBinding::DEFAULT).
			 addWorldStatePredicate(new IsPlayOnMode( true ) ).
			 addWorldStatePredicate(new IsNearBall(true)));


  /* - Action: Intercept; Source: Default;
     
     World State Predicate: Agent is defending our goal
  */
  listOfLevels.push_back(Level(ActionVisitor::INTERCEPT,
			       ActionBinding::DEFAULT).
			 addWorldStatePredicate(new IsPlayOnMode( true ) ).
			 addWorldStatePredicate(new IsBallNearOurGoal(true)).
			 addWorldStatePredicate(new IsCloseToOurGoal(true)));


  /* - Action: Intercept; Source: Default;

     World State Predicate: Agent is closest to the ball
  */
  listOfLevels.push_back(Level(ActionVisitor::INTERCEPT,
			       ActionBinding::DEFAULT).
			 addWorldStatePredicate(new IsPlayOnMode( true ) ).
			 addWorldStatePredicate(new IsClosestToBall(true ) ) );
  

  /* - Action: Intercept; Source: Coach; Predicates: None */
  listOfLevels.push_back(Level(ActionVisitor::INTERCEPT,ActionBinding::COACH));

  /* - Action: Oline; Source: Coach; Predicates: None */
  listOfLevels.push_back(Level(ActionVisitor::OFFSIDESLINE,
			       ActionBinding::COACH));


  /* - Action: Mark; Source: Coach;
     Action Predicate: Opponent is unmarked */
  listOfLevels.push_back(Level(ActionVisitor::MARK,ActionBinding::COACH).
			 addActionPredicate(new KnowsOpponentPos <ActMark> 
					    ( true )).
			 addActionPredicate(new OpponentIsMarkedByTeammate
					    ( false )));


  /* - Action: MarkLinePlayer; Source: Coach;
     Action Predicate: Opponent is unmarked */
  listOfLevels.push_back(Level(ActionVisitor::MARKLINEPLAYER,
			       ActionBinding::COACH).
			 addActionPredicate(new KnowsOpponentPos 
					    <ActMarkLinePlayer> (true)).
			 addActionPredicate(new OpponentIsMarkedLineByTeammate
					    (false)));
			 

  /* - Action: Pos; Source: Coach; Predicates: None */
  listOfLevels.push_back(Level(ActionVisitor::POS,ActionBinding::COACH));

  /* - Action: MarkLinePlayer; Source: Coach;
     Action Predicate: Opponent is close to the ball */
  listOfLevels.push_back(Level(ActionVisitor::MARKLINEPLAYER, 
			       ActionBinding::COACH).
			 addActionPredicate(new KnowsOpponentPos 
					    <ActMarkLinePlayer> (true)).
			 addActionPredicate(new OpponentCloseToBall
					    <ActMarkLinePlayer> (true)));
			 


  /* - Action: Mark; Source: Coach;
     Action Predicate: Opponent is close to the ball */
  listOfLevels.push_back(Level(ActionVisitor::MARK,ActionBinding::COACH).
			 addActionPredicate(new KnowsOpponentPos <ActMark> 
					    (true)).
			 addActionPredicate(new OpponentCloseToBall
					    <ActMark> ( true )));
			 


  /* - Action: Mark; Source: Coach; */
  listOfLevels.push_back(Level(ActionVisitor::MARK,ActionBinding::COACH).
			 addActionPredicate(new KnowsOpponentPos <ActMark> 
					    (true)));


  /* - Action: MarkLinePlayer; Source: Coach; */
  listOfLevels.push_back(Level(ActionVisitor::MARKLINEPLAYER,
			       ActionBinding::COACH).
			 addActionPredicate(new KnowsOpponentPos 
					    <ActMarkLinePlayer> ( true )));

  
  /* - Action: Intercept; Source: Rulefile */
  listOfLevels.push_back(Level(ActionVisitor::INTERCEPT,
			       ActionBinding::RULEFILE));
  
  /* - Action: Tackle; Source: Rulefile */
  listOfLevels.push_back(Level(ActionVisitor::TACKLE,
			       ActionBinding::RULEFILE));

  /* - Action: Oline; Source: Rulefile */
  listOfLevels.push_back(Level(ActionVisitor::OFFSIDESLINE,
			       ActionBinding::RULEFILE));

  /* - Action: Mark; Source: Rulefile */
  listOfLevels.push_back(Level(ActionVisitor::MARK,
			       ActionBinding::RULEFILE).
			 addActionPredicate(new OpponentIsMarkedByTeammate
					    (false)).
			 addActionPredicate(new KnowsOpponentPos 
					    <ActMark> (true)));


  /* - Action: MarkLinePlayer; Source: Rulefile */
  listOfLevels.push_back(Level(ActionVisitor::MARKLINEPLAYER,
			       ActionBinding::RULEFILE).
			 addActionPredicate(new OpponentIsMarkedLineByTeammate
					    (true)).
			 addActionPredicate(new KnowsOpponentPos 
					    <ActMarkLinePlayer> 
					    (true)));



  /* - Action: Pos; Source: Rulefile */
  listOfLevels.push_back(Level(ActionVisitor::POS,ActionBinding::RULEFILE));

}



/* Tests if the given action can be classified into the given level */
bool LevelStorage::matchLevel(const ActionBinding &abinding, 
			      const int &level) const
{
  /* First retrieve the current level in consideration from the storage */
  Level currLevel = listOfLevels[level];
  
  /* Retrieve the action from the action binding */
  Action *act = abinding.theaction;

  /* Retrieve the VarBindings for the action */
  VarBindings v = abinding.myBindings;


  /* Utilize short-circuit evaluation here:
     
     1) match the world state
     2) match the source of the action advice
     3) match the action type
     4) satisfy all the action specific properties of the level
   */
  return ( currLevel.matchWorldState() &&
	   currLevel.matchSourceType(abinding.sourceType) &&
	   currLevel.matchActionType(act) &&
	   currLevel.matchAction(act, v) );
}
  
/* Clear all listOfLevels */
void LevelStorage::clear()
{
  listOfLevels.clear();
}




