/* Dongryeol Lee
 *
 * LevelStorage.h
 *
 * This is the header class definition for storing all the user-defined
 * levels for the action selection algorithm. It also provides a way to
 * test if the given action fits into the given level
 *
 */


#ifndef LEVEL_STORAGE
#define LEVEL_STORAGE


#include "Level.h"
#include <vector>
#include "clangaction.h"


class LevelStorage
{
 private:
  
  /* Store all of our levels here */
  std::vector <Level> listOfLevels;

 public:

  /* Method to add a Level to our storage */
  void addLevel(const Level &newlevel)
    {
      listOfLevels.push_back(newlevel);
    }

  /* Initialize the storage with default sets of levels */
  void init();

  /* Constructor */
  LevelStorage() {
    /* pfr 2/17/2004: BIG HACK
       Because there is so much global variable crap throughout the code,
       it's hard to control when things get initialized
       I removed the init call here and let CoachablePlayer do it
       since we know when that is going to happen */
  }

  /* Method to retrieve how many levels are in our storage */
  const int getNumLevels() const { return listOfLevels.size(); }

  /* Tests if the given action can be classified into the given level */
  bool matchLevel(const ActionBinding &abinding, const int &level) const;

  /* Method to retrieve a Level object stored in our storage */
  Level getLevel(const int &levelno) const { return listOfLevels[levelno]; }

  /* Method to retrieve the source of a given Level in our storage */
  ActionBinding::SourceType getSourceType(const int &levelno) const
    { return listOfLevels[levelno].getSourceType(); }

  /* Method to retrieve the action type of a given Level in our storage */
  ActionVisitor::ActionType getActionType(const int &levelno) const
    { return listOfLevels[levelno].getActionType(); }
  
  /* Clear all levels */
  void clear();

};


#endif
