/*
Copyright (c) 2000-2002, Jelle Kok, University of Amsterdam
All rights reserved.

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this 
list of conditions and the following disclaimer. 

2. Redistributions in binary form must reproduce the above copyright notice, 
this list of conditions and the following disclaimer in the documentation 
and/or other materials provided with the distribution. 

3. Neither the name of the University of Amsterdam nor the names of its 
contributors may be used to endorse or promote products derived from this 
software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE 
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
/*! \file Player.h
<pre>
<b>File:</b>          Player.h
<b>Project:</b>       Robocup Soccer Simulation Team: UvA Trilearn
<b>Authors:</b>       Jelle Kok
<b>Created:</b>       03/03/2001
<b>Last Revision:</b> $ID$
<b>Contents:</b>      This file contains the declaration for the Player class,
               which is a superclass from BasicPlayer and contains the
               decision procedure to select the skills from the
               BasicPlayer. 
<hr size=2>
<h2><b>Changes</b></h2>
<b>Date</b>             <b>Author</b>          <b>Comment</b>
03/03/2001       Jelle Kok       Initial version created
</pre>
*/

#ifndef PLAYER
#define PLAYER

#include "CoachablePlayer.h"
#include "Formations.h"     // needed for Formations
#include "PlayerInterface.h"
#include "CoachableAgentComm.h"

void* stdin_callback( void * v );


/*! This class is a superclass from BasicPlayer and contains a more
    sophisticated decision procedure to determine the next action. */
class Player : public CoachablePlayer 
{
  Formations    *formations;             /*!< all formation information      */
  bool          bContLoop;               /*!< is server is alive             */

  Time          m_timeLastSay;           /*!< last time communicated         */

  PlayerInterface *Player_interface;
  CoachableAgentComm *AgentComm;    // communication class for sending say msg's
  ofstream       *logpos;            // stream for logging player+teammate positions
  ofstream       *statslog;         // holds file stream if we're logging WM stats


  // private variables for logging world model stats: 
  int totalcycles;      // total cycles we've logged
  int ballconfcycles;   // total cycles we're confident of ball
  int iTeammatecycles[11]; // number of cycles we're confident of each teammate
  int iOpponentcycles[11]; // number of cycles we're confident of each opponent
  // Error variables: 
  float fBallposError;  // the error between where we think ball is and where it 
                        // actually is based on the fullstate server data.
  float fTeammateError[11]; // error in teammate pos's compared to actual pos.
  float fOpponentError[11]; // error in opponent pos's compared to actual pos.


  // private functions: 
  void          goalieMainLoop            (                                   );
  void          defenderMainLoop          (                                   );
  void          midfielderMainLoop        (                                   );
  void          attackerMainLoop          (                                   );

  // methods associated with saying (defined in Player.C)
  bool          shallISaySomething        (                                   );
  bool          amIAgentToSaySomething    (                                   );
  SoccerCommand sayBallStatus             (                                   );
  SoccerCommand sayCoachableAgentMsg( );
  void          changeAttentionto();
  // player position logging function:
  void          logPositions();
  
  void          logWorldModelStats();

public:
  Player                                  ( ActHandler     *a,
                                            WorldModel     *wm,
                                            ServerSettings *ss,
                                            PlayerSettings *cs,
                                            Formations     *fs,
					    PlayerInterface *pi,
					    SenseHandler   *sense,
					    CoachableAgentComm *comm,
					    ofstream        *positionlog,
                                            char           *strTeamName,
                                            double         dVersion,
                                            int            iReconnect = -1,
					    bool           useGUI = false,
					    ofstream       *WMstatslog = NULL
					   );

  void          mainLoop                  (                                   );
  void          deMeer5                   (                                   );
  void          deMeer5_goalie            (                                   );

  // methods that deal with user input (from keyboard) to sent commands
  void          handleStdin               (                                   );
  void          showStringCommands        ( ostream& out                      );
  bool          executeStringCommand      ( char *str                         );

  // test methods
  void          test_only_update          (                                   );
} ;


#endif
