/*
 * A visitor class to handle Msg types. 
 *
 * John Davin
 */


#ifndef RULE_VISITOR_H
#define RULE_VISITOR_H

#include "visitor.h"
#include "rule.h"
#include "hasa.h"
#include "coach_lang_comp.h"
#include "config.h"
#include "VarBindings.h"
#include "ActionBinding.h"


class RuleVisitor : public Rule::Visitor
{

 private: 

  /* Variable binding */
  VarBindingSet myBindings;
  
  /* Source of the rule: passed along to ActionPicker eventually */
  ActionBinding::SourceType sourceType;

  /* holds the name of this rule. This is *only* set if we are logging the matching rules */
  /* msgrulestore.C will initialize the RuleVisitor with a rulename if we are logging */
  std::string rulename;
  int rulelogging;// 0=no rule logging, 1=log names of matched rules, 2=log contents of matched rules

  bool force_cond_true; // if on, all conditions are considered to evaluate to true
  
 public:
  typedef std::vector<Dir*>::iterator DirIter;
  
  RuleVisitor(VarBindingSet b, ActionBinding::SourceType source, std::string name, int ruleloglevel, bool force_cond_true = false)
    : myBindings(b), sourceType(source), rulename(name), rulelogging(ruleloglevel), force_cond_true(force_cond_true)
    {}
  
  RuleVisitor(ActionBinding::SourceType source, bool force_cond_true = false)
    : sourceType(source), rulename(""), rulelogging(0), force_cond_true(force_cond_true)
    {}


  RuleVisitor(ActionBinding::SourceType source, std::string name, int ruleloglevel, bool force_cond_true = false)
    : sourceType(source), rulename(name), rulelogging(ruleloglevel), force_cond_true(force_cond_true)
    {}

  ~RuleVisitor() {}
  
  void visit( SimpleRule *rule);
  
  void visit( NestedRule *rule);

  void visit( CondRule *rule);

  void visit( IDListRule *rule);

  void setForceCondTrue(bool b) { force_cond_true = b; }

};


#endif // RULE_VISITOR_H
