/*
Copyright (c) 2000-2002, Jelle Kok, University of Amsterdam
All rights reserved.

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this 
list of conditions and the following disclaimer. 

2. Redistributions in binary form must reproduce the above copyright notice, 
this list of conditions and the following disclaimer in the documentation 
and/or other materials provided with the distribution. 

3. Neither the name of the University of Amsterdam nor the names of its 
contributors may be used to endorse or promote products derived from this 
software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE 
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
/*! \file main.C
<pre>
<b>File:</b>          main.C
<b>Project:</b>       Robocup Soccer Simulation Team: UvA Trilearn
<b>Authors:</b>       Jelle Kok
<b>Created:</b>       28/11/2000
<b>Last Revision:</b> $ID$
<b>Contents:</b>      This file contains the main of the program that is used
               to start the agent. It creates all classes, starts the different 
               threads and calls the mainloop. Furthermore it parses the
               specified arguments to adjust the variables.
<hr size=2>
<h2><b>Changes</b></h2>
<b>Date</b>             <b>Author</b>          <b>Comment</b>
28/11/2000       Jelle Kok       Initial version created
</pre>
*/

#include "LevelStorage.h"
#include "SenseHandler.h"
#include "Player.h"
#include "Parse.h"
#include "PlayerInterface.h"
#include "PlayerTagFunction.h"
#include <string.h>   // needed for strcpy
#include <pthread.h>  // needed for pthread_create
#include <stdlib.h>   // needed for exit
#include <sstream>


Player *MY_PLAYER;
PlayerInterface *PI; // used in SenseHandler.C
extern Logger Log; /*!< This is a reference to the Logger to write log info to*/

/* logging stream for matched rules */
ofstream rulelog;



void printOptions( );

/*! This is the main function and creates and links all the different classes.
    First it reads in all the parameters from the command prompt
    (<program name> -help) and uses these values to create the classes. After 
    all the classes are linked, the mainLoop in the Player class is called. */
int main( int argc, char * argv[] )
{
  pthread_t      listen, sense;
  ServerSettings* pss = new ServerSettings;
  PlayerSettings* pcs = new PlayerSettings;

  // define variables for command options and initialize with default values
  char     strTeamName[MAX_TEAM_NAME_LENGTH] = "Carnegie";
  int      iPort                             = pss->getPort();
  int      iActionLogLevel                   = -1; //200;
  int      iMinLogLevel                      = 0;
  int      iMaxLogLevel                      = 0;  
  char     strHost[128]                      = "localhost";
  double   dVersion                          = 8.08;
  int      iMode                             = 0;
  char     strFormations[128]                = "formations.conf";
  int      iNr                               = -1;
  int      iReconnect                        = -1;
  bool     bInfo                             = false;
  bool     bSuppliedLogFile                  = false;
  bool     cmdline                           = false;
  bool     useGUI                            = false;
  int      rulelogging                       = 0; 
  // rulelogging: 0=no logging,1=rule name logging,2=rule contents logging
  bool     stdLoggingOn                      = false;

  /* logging stream for player position logging */
  ofstream *logpos = NULL;
  /* logging stream for player position logging */
  ofstream *statslog = NULL;
  
  char *coachfile = NULL;
  ofstream os;
  std::ostringstream rulelogfile;
  std::ostringstream logposfile;
  std::ostringstream statsfile;

  // read in all the command options and change the associated variables
  // assume every two values supplied at prompt, form a duo
  char * str;
  for( int i = 1 ; i < argc ; i = i + 2  )
  {
    // make sure this option has an argument (unless it is one of the options
    // that don't require a parameter). 
    if( i + 1 >= argc && strncmp( argv[i], "-help", 3 ) != 0 
	&& strncmp( argv[i], "-cmdline", 3) != 0 && strncmp(argv[i], "-gui", 2)
	&& strncmp( argv[i], "-stats", 3) )
    {
      cout << "Need argument for option: " << argv[i] << endl;
      printOptions();
      exit( 0 );
    }
    // read a command option
    if( argv[i][0] == '-' && strlen( argv[i] ) > 1)
    {
      switch( argv[i][1] )
      {
        case '?':                                   // print help
          printOptions( );
          exit(0);
          break;
        case 'c':                                   // clientconf file or cmdline
          if( strlen( argv [i]) > 2 && argv[i][2] == 'm' ){ // -cmdline
	    cmdline = true;
	    i--; // so i is at correct position next iteration (cmdline doesn't have a second arg).
	    break;
	  }
	  else if(strlen( argv [i]) > 2 && argv[i][2] == 'f'){ // -cf(ile) - load coach cmd file
	    coachfile = &argv[i+1][0];
	    break;
	  }

          if( pcs->readValues( argv[i+1], ":" ) == false )
            cerr << "Error in reading client file: " << argv[i+1] << endl;
          break;
        case 'f':                                   // formations file
          strcpy( strFormations, argv[i+1] );
          break;
        case 'h':                                   // host server or help
          if( strlen( argv [i]) > 2 && argv[i][2] == 'e' )
          {
            printOptions( );
            exit(0);
          }
          else
            strcpy( strHost, argv[i+1] );
          break;
        case 'i':                                   // info 1 0
          str   = &argv[i+1][0];
          bInfo = (Parse::parseFirstInt( &str ) == 1 ) ? true : false ;
          break;
        case 'a':                                   // 
          str   = &argv[i+1][0];
	  iActionLogLevel = Parse::parseFirstInt( &str );
          break;
        case 'l':                                   // loglevel int[..int] OR -logpos
          str = &argv[i+1][0];

          if( strlen( argv [i]) >= 5 && argv[i][4] == 'p' ){ // -logpos
	    logpos = new ofstream;
	    // str contains directory name
	    logposfile << str; 
	    break;
	  }

	  // otherwise, set up logging level: 
	  stdLoggingOn = true;
          iMinLogLevel = Parse::parseFirstInt( &str );
          while( iMinLogLevel != 0 )
          {
            if( *str == '.' || *str == '-') // '.' or '-' indicates range 
            {
              str++;
              iMaxLogLevel = Parse::parseFirstInt( &str );            
              if( iMaxLogLevel == 0 ) iMaxLogLevel = iMinLogLevel;
              Log.addLogRange( iMinLogLevel, iMaxLogLevel );
            }
            else
              Log.addLogLevel( iMinLogLevel );
            iMinLogLevel = Parse::parseFirstInt( &str );                          
          }
          break;
        case 'm':                                   // mode int
          str = &argv[i+1][0];
          iMode = Parse::parseFirstInt( &str );
          break;
        case 'n':                                   // number in formation int
          str = &argv[i+1][0];
          iNr = Parse::parseFirstInt( &str );
          break;
        case 'o':                                   // output file log info
          os.open( argv[i+1] );
          bSuppliedLogFile = true;
          break;
        case 'p':                                   // port
          str = &argv[i+1][0];
          iPort = Parse::parseFirstInt( &str );
          break;
        case 'r':                                   // reconnect 1 0
          str = &argv[i+1][0];
          iReconnect = Parse::parseFirstInt( &str );
          break;
        case 's':                                   // serverconf file or -stats

          if( strlen( argv [i]) >= 3 && argv[i][2] == 't' ){ // -stats <dir>
	    str = &argv[i+1][0]; // directory name
	    statslog = new ofstream;
	    statsfile << str; 
	    break;
	  }

          if( pss->readValues( argv[i+1], ":" ) == false )
            cerr << "Error in reading server file: " << argv[i+1] << endl;
          break;
        case 't':                                   // teamname name
          strcpy( strTeamName, argv[i+1] );
          break;
        case 'v':                                   // version version
          str = &argv[i+1][0];
          dVersion = Parse::parseFirstDouble( &str );
          break;
        case 'g':                                   // use Actions GUI
	  useGUI = true;
	  i--; // so i is at correct position next iteration (useGUI doesn't have a second arg).
          break;
      case 'z':
	rulelogging = 1; // turns on rule name logging 
	if( strlen( argv [i]) > 2 && argv[i][2] == '2' ){ // -z2
	  rulelogging = 2; // print rule expressions instead of names
	}
	
	str = &argv[i+1][0]; // specifies the dir (eg, use "." for current dir)

	rulelogfile << str; 

	break;

      default:
	cerr << "(main) Unknown command option: " << argv[i] << endl;
	break;
      }
    }
  }

  if( bInfo == true )
    cout << "team         : "  << strTeamName    << endl <<
            "port         : "  << iPort          << endl <<
            "host         : "  << strHost        << endl <<
            "version      : "  << dVersion       << endl <<
            "mode         : "  << iMode          << endl <<
            "playernr     : "  << iNr            << endl <<
            "reconnect    : "  << iReconnect     << endl ;

  if(iNr == -1){ // player num was not specified.
    iNr = 2; // default to 2.

    // if any of the logs are turned on, print a warning because we will 
    // be logging to a file name that may have a different number than the actual
    // player number that the player will get assigned.
    if(stdLoggingOn || iActionLogLevel != -1 || logpos != NULL || rulelogging != 0)
      cerr << "Warning: You did not specify a player number (with -n) so log files \n"
	   << "will default to playernum="<<iNr<<", which may be different from \n"
	   << "the number the player gets assigned by the server."<<endl;

  }

 
  if( bSuppliedLogFile == true ){
    Log.setOutputStream( os );                   // initialize logger
  }
  else{

    if(stdLoggingOn){
      std::ostringstream fn;
      fn << "Logfiles/player" << iNr << ".log" << ends;
      os.open(fn.str().c_str());
      if (!os){
	cerr << "Could not open player log (" << fn.str()<<"), using stdout." << endl;
	Log.setOutputStream( cout );
      }
      else{
	Log.setOutputStream( os );
      }
    }

  }

  // if -logpos was specified, open file stream for player position logging
  if(logpos != NULL) { 
    // the directory name is already in logposfile
    logposfile << "/" << iNr << "-playerpos.log" << ends;
    logpos->open(logposfile.str().c_str());
    if (!logpos->is_open()){
      cerr << "Not writing to position log file. Could not open: " 
	   << logposfile.str() << endl;
      delete logpos;
      logpos = NULL;
    }
  }

  // if -z was specified (for rulelogging), open file stream for matched rules log
  if(rulelogging != 0){
    // the directory name is already in rulelogfile
    rulelogfile << "/" << iNr << "-matched.log" << ends;
    rulelog.open(rulelogfile.str().c_str());
    if (!rulelog){
      cerr << "Not writing to matched rules log file. Could not open: " 
	   << rulelogfile.str() << endl;
      rulelogging = 0;
    }
  }

  // if -stats was specified, open file stream for world model stats logging
  if(statslog != NULL) { 
    // the directory name is already in statsfile
    statsfile << "/" << iNr << "-stats.log" << ends;
    statslog->open(statsfile.str().c_str());
    if (!statslog->is_open()){
      cerr << "Not writing to stats log file. Could not open: " 
	   << statsfile.str() << endl;
      delete statslog;
      statslog = NULL;
    }
  }

  Log.restartTimer( );


  Formations* pfs = new Formations( strFormations, FT_433_OFFENSIVE, iNr-1 );// read formations file
  WorldModel* pwm = new WorldModel( pss, pcs, pfs );              // create worldmodel

  // communication class (passed to Player and SenseHandler)
  CoachableAgentComm *comm = new CoachableAgentComm(pwm);
  pwm->setAgentComm(comm);

  Connection* pc = new Connection( strHost, iPort, MAX_MSG );     // make connection with server
  ActHandler* pa = new ActHandler( pc, pwm, pss );  
  SenseHandler* ps = new SenseHandler( pc, pwm, pss, pcs, comm ); 

  PlayerTagFunction *pTagFunc = new PlayerTagFunction(pwm);

  if(iActionLogLevel != -1){
    spades::Logger::instance()->setTagFunction(pTagFunc);
    spades::Logger::instance()->setMaxActionLogLevel(iActionLogLevel);
    std::ostringstream fn;
    fn << "actionlog_p" << iNr << ".log" << ends;
    spades::Logger::instance()->openFiles("Logfiles", fn.str());
  }

  //actionlog(100) << "This is a test" << spades::ende;
  //errorlog << "Err test" << spades::ende;
  
  // Since the Player spec has changed now, PlayerInterface is being integrated. 
  // It's used by the player to evaluate coach commands now. 
  PlayerInterface* ppi = new PlayerInterface(pwm, cmdline, coachfile, rulelogging);
  Player* pbp = new Player( pa, pwm, pss, pcs, pfs, ppi, ps, comm, logpos, 
			    strTeamName, dVersion, iReconnect, useGUI, statslog );
                                               // create player
  PI = ppi;
  MY_PLAYER = pbp; // set a pointer to the player	
  
  pthread_create( &sense, NULL, sense_callback  , ps); // start listening

  if( iMode > 0 ) // only listen to standard input when not playing
    pthread_create( &listen, NULL, stdin_callback, pbp); 

  if( iMode == 0 )
    pbp->mainLoop();
  else if( iMode == 1 )
    pbp->test_only_update();

  pc->disconnect();
  os.close();
  rulelog.close();
  if(logpos != NULL) logpos->close();
  if(statslog != NULL) statslog->close();

  pTagFunc->forgetWorldModel();

  if(logpos != NULL) delete logpos;
  if(statslog != NULL) delete statslog;
  delete comm;
  delete pbp;
  delete ppi;
  delete ps;
  delete pa;
  delete pc;
  delete pwm;
  delete pfs;
  delete pcs;
  delete pss;
}

/*! This function prints the command prompt options that can be supplied to the
    program. */
void printOptions( )
{
  cout << "Command options:"                    << endl << 
    " a(ctionloglevel) int  - level of secondary logging info"        << endl <<                    
    " c(lientconf) file     - use file as client conf file"           << endl <<
    " cm(dline)             - use stdin for inputting coach commands" << endl <<
    " cf(ile)               - specify a coach command file"           << endl <<
    " f(ormations) file     - file with formation info"               << endl <<
    " g(ui)                 - use Actions GUI"                        << endl <<
    " he(lp)                - print this information"                 << endl <<
    " h(ost) hostname       - host to connect with"                   << endl <<
    " i(nfo) 0/1            - print variables used to start"          << endl <<
    " l(oglevel) int[..int] - level of debug info"                    << endl <<
    " logpos dir            - log player positions to specified directory"<< endl <<

    " m(ode) int            - which mode to start up with"            << endl <<
    " n(umber) int          - player number in formation"             << endl <<
    " o(utput) file         - write log info to (screen is default)"  << endl <<
    " p(ort)                - port number to connect with"            << endl <<
    " r(econnect) int       - reconnect as player nr"                 << endl <<
    " s(erverconf) file     - use file as server conf file"           << endl <<
    " stats dir             - log and print world model statistics"   << endl <<
    " t(eamname) name       - name of your team"                      << endl <<
    " z(1|2) dir            - turn on rule logging: "<<endl<<
    "                         1=rule names, 2=rule expressions" << endl;
}
