/*
 * A visitor class to handle Msg types. 
 *
 * John Davin
 */


#ifndef MY_MSG_SORTER_H
#define MY_MSG_SORTER_H

#include "visitor.h"
#include "clangmsg.h"
#include "clangdefmsg.h"
#include "clangrulemsg.h"
#include "clangdelmsg.h"
#include "hasa.h"
#include "config.h"
#include "msgsorterdefine.h"
#include "ActionBinding.h"


class MyMsgSorter : public Msg::Visitor
{
 private: 

  /* Source of the message */
  ActionBinding::SourceType sourceType;

 public:

  MyMsgSorter(ActionBinding::SourceType source)
    : sourceType(source)
    {}

  /* When we get a define msg, we use another visitor on it to figure 
   * out which type of define it is. */
  void visit( DefineMsg* msg ){
    PR(4, "Got Define message\n");
    
    // a define message can be composed of multiple objects, so we need to get the tokens
    util::HasMany< std::auto_ptr<Def> >::Storage deftokens = msg->getTokens();

    std::list<Def*>::iterator i = deftokens.begin();
    while(i != deftokens.end()){

      // use another Visitor to handle each of the subtypes of a DefineMsg. 
      MsgSorterDefine mysorter(sourceType);
      (*i)->accept(mysorter);
    
      i++;
    }

  }

  /* Handle Rule messages (for example, by turning on whatever rules it specifies) */
  void visit( RuleMsg* msg ){
    PR(4, "Got Rule Message\n");

    // get the list of rules
    util::HasMany< ActivateRules >::Storage items = msg->getList();
    
    std::list< ActivateRules >::iterator it = items.begin();

    ActivateRules ruleslist;

    while(it != items.end()){

      ruleslist = *it;
      if(ruleslist.on()){ // turn rules on 
	if(ruleslist.all()){ // turn *all* the rules on
	  PR(6, "[MyMsgSorter]: Turning all rules on");
	  
	  defrulestorage.TurnOnAll();

	}
	else{ // turn on the rules that are specified

	  RuleIDList rulenamelist = ruleslist.getList();

	  std::list< RuleID >::iterator rulenames = rulenamelist.begin();
	  while(rulenames != rulenamelist.end()){
	    defrulestorage.TurnOn(*rulenames);
	    rulenames++;
	  }

	}
      }
      else{ // turn rules off
	if(ruleslist.all()){ // turn *all* the rules off
	  PR(6, "[MyMsgSorter]: Turning all rules off");

	  defrulestorage.TurnOffAll();
	}
	else{ // turn off the rules that are specified
	  RuleIDList rulenamelist = ruleslist.getList();

	  std::list< RuleID >::iterator rulenames = rulenamelist.begin();
	  while(rulenames != rulenamelist.end()){
	    defrulestorage.TurnOff(*rulenames);
	    rulenames++;
	  }
	}
      }

      it++;
    }

  }

  /* delete definitions */
  void visit( DelMsg* msg ){
    int i;
    /* Note: if you delete a rule, you must delete it from onrules also - so that we
     * don't leave a bad pointer in there. */

    RuleIDList rulelist = msg->getList();

    for(std::list< RuleID >::iterator rulenames = rulelist.begin(); 
	rulenames != rulelist.end(); rulenames++){

      i= defrulestorage.remove(*rulenames);
      if(!i) PR(9, "[DelMsg]: ERROR: failed to delete rule %s\n", (*rulenames).c_str());
      else   PR(4, "[DelMsg]: Deleted rule %s\n", (*rulenames).c_str());

    }

  }



  void visit( MetaMsg* ){
    PR(6, "MetaMsg not yet supported\n");
  }  
  void visit( InfoMsg* ){
    PR(6, "InfoMsg not supported\n");
  }
  void visit( AdviceMsg* ){
    PR(6, "AdviceMsg not supported\n");
  }
  void visit( FreeformMsg* ){
    PR(6, "Freeform messages are unsupported\n");
  }
  void visit( UnsuppMsg* ){
    PR(6, "Received unsupported message!\n");
  }

};


#endif // MY_MSG_SORTER_H
