/*
    Coachable Agent communication encoding and decoding library
    Copyright (C) 2004, John Davin, Carnegie Mellon University

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

/*
 * charConversionHelper.C - contains some simple character conversion functions 
 *  which are used by SayMsgTypes.h and SayMsgDecoder. 
 *
 *
 * Creation Date: 11/18/03
 * Author: John Davin
 *
 *
 */

#include <iostream>
#include <assert.h>
using namespace std;
#include "charConversionHelper.h"



// char2numMap holds the char->number mapping for validchars[] 
// (ie, it is the inverse of validchars)
short char2numMap[256] = {-2}; 
// just make it size of ascii char range, even though we won't need that much space




/*
 * returns the index into the validchars array for char c 
 * Returns -1 on error (eg, char c is not in validchars)
 */
short get_num_from_char(char c){
  /* if the char2numMap is not computed yet, then we must do so now */
  /* (this part will only be run once) */
  if(char2numMap[0] == -2){

    for(int i=0; i<256; i++)
      char2numMap[i] = -1;
    
    for(int i=0; i<NUMCHARS; i++){
      char2numMap[(int)validchars[i]] = i;
    }
  }

  return char2numMap[(int)c];
}



/* helper function for SayMsgTypes::getEncodedStr(): 
 * Given i, an index into validchars array, returns the appropriate char. 
 * If i is out of bounds, rounds to nearest bound. 
 */
char get_printable_char(float i){
  // cast i to integer using floor (standard C casting) as defined in specification
  int index = (int)i;
  if(index < 0){
    cerr << "[get_printable_char]: warning: "<<index<<" was below bounds"<<endl;
//#warning Remove assert(0) for non-debug compiles
    //assert(0);
    return validchars[0];
  }
  else if(index >= NUMCHARS){
    cerr << "[get_printable_char]: warning: "<<index<<" was above bounds"<<endl;
//#warning Remove assert(0) for non-debug compiles
    //assert(0);
    return validchars[NUMCHARS-1];
  }
  else { // index is within bounds

    return validchars[index];
  }
    
}
