/*
 * main.C - Simple utility program that you invoke with an encoded 
 *  say message string on the command line and it prints (to stdout)
 *  a human-readable representation of the message. 
 *  (you can pipe or redirect the output to use it however you wish - 
 *   eg, for logging to a file). 
 *
 * Creation Date: 12/3/03
 * Author: John Davin
 *
 *
 *
 */

#include <iostream>
#include <fstream>
using namespace std;

#include "SayMsgEncoder.h"
#include "SayMsgDecoder.h"
#include "Logging_SayMsgFactory.h"

void printUsage();

// global string that the Logging_SayMsgTypes will use to store the text string
// that the msg type translates to (in process()). 
// The main loop will retrieve this string from this variable. 
stringstream ProcessedStr;



string decodeMsg(string theSayMsg){
  string str("");

  SayMsgFactory *msgfactory = new Logging_SayMsgFactory();
  SayMsgDecoder *decoder = new SayMsgDecoder(msgfactory);
  bool success = decoder->decodeStr(theSayMsg, 2); //playernum doesn't matter
  if(!success){ 
    cout << "Failed to decode message: "<<theSayMsg<<endl;
    return str;
  }
  SayMsgDecoder::DecodedMsgIterator iter = decoder->getMsgIterator();
  for( ; iter != decoder->getMsgIteratorEnd(); iter++){
    SayMsgTypes *curunit = *iter;
    curunit->process();
    string result = ProcessedStr.str();
    ProcessedStr.str(""); // clear the stream

    // chop off the /n at the end
    result.resize(result.size()-1);

    if(str == "")
      str += result;
    else
      str += ", " + result;
  }

  return str;
}

/*
 * usage: ./a.out <encoded msg string>
 *
 */
int main(int argc, char *argv[]){
  char line[200];
  ifstream rclfile;
  int time;
  char player[50];
  char saymsg[50];
  int n;

  if(argc != 2)
    printUsage();

  rclfile.open(argv[1]);

  if(! rclfile.is_open()){
    cerr << "[decodelog]: failed to open rcl logfile "<<argv[1]<<endl;
    exit(1);
  }

  while(! rclfile.eof()){
    rclfile.getline(line, 200);

    n = sscanf(line, "%d\tRecv %s (say %50s", &time, player, saymsg);
    if(n != 3){
      //cerr << "sccanf failed on: "<<line<<endl;
      continue;
    }

    // saymsg will have a ) at the end
    if(saymsg[strlen(saymsg)-1] != ')')
      continue;
    saymsg[strlen(saymsg)-1] = '\0';

    // remove : from end of player name: 
    player[strlen(player)-1] = '\0';

    string theSayMsg = saymsg;
    string decoding = decodeMsg(theSayMsg);

    cout << time <<": "<<player<< " said: "<<decoding<<endl;

  }

  return 0;
}



void printUsage(){
  cout << "Usage: ./a.out <rcl log file>"<<endl;
  exit(1);

}

