/*
 * main.C - Simple utility program that you invoke with an encoded 
 *  say message string on the command line and it prints (to stdout)
 *  a human-readable representation of the message. 
 *  (you can pipe or redirect the output to use it however you wish - 
 *   eg, for logging to a file). 
 *
 * Creation Date: 12/3/03
 * Author: John Davin
 *
 *
 *
 */


#include "SayMsgEncoder.h"
#include "SayMsgDecoder.h"
#include "Logging_SayMsgFactory.h"

void printUsage();

// global string that the Logging_SayMsgTypes will use to store the text string
// that the msg type translates to (in process()). 
// The main loop will retrieve this string from this variable. 
stringstream ProcessedStr;


/*
 * usage: ./a.out <encoded msg string>
 *
 */
int main(int argc, char *argv[]){
  
  if(argc != 2)
    printUsage();


  string theSayMsg(argv[1]);
  SayMsgFactory *msgfactory = new Logging_SayMsgFactory();
  SayMsgDecoder *decoder = new SayMsgDecoder(msgfactory);
  bool success = decoder->decodeStr(theSayMsg, 2); //playernum doesn't matter
  if(!success){ 
    cout << "Failed to decode message: "<<theSayMsg<<endl;
    return 1;
  }
  SayMsgDecoder::DecodedMsgIterator iter = decoder->getMsgIterator();
  for( ; iter != decoder->getMsgIteratorEnd(); iter++){
    SayMsgTypes *curunit = *iter;
    curunit->process();
    cout << ProcessedStr.rdbuf();
  }


  return 0;
}



void printUsage(){
  cout << "Usage: ./a.out <encoded msg string>"<<endl;
  exit(1);

}

