/*
    Coachable Agent communication encoding and decoding library
    Copyright (C) 2004, John Davin, Carnegie Mellon University

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

/*
 * SampleSayMsgTypes.h - Defines subclasses of the SayMsgTypes in order to 
 *  define our process() functions (which can add data to the world model)
 *
 * Creation Date: 11/18/03
 * Author: John Davin
 *
 *
 */

#ifndef SAMPLE_SAY_MSG_TYPES_H
#define SAMPLE_SAY_MSG_TYPES_H

#include "SayMsgTypes.h"
#include <sstream>
using namespace std;

extern stringstream ProcessedStr; //from main.C


/*
 * OurPos member variables: 
 * float X - our position X value 
 * float Y - our position Y value
  */
class MyOurPos : public OurPos {

 public:

  void process(){
    ProcessedStr << "OurPos("<<X<<","<<Y<<")"<<endl;

  }

};

/*
 * BallPos member variables: 
 * float X - ball position X value 
 * float Y - ball position Y value
 * unsigned short cycleno - number of cycles since saw ball
 */
class MyBallPos : public BallPos {

 public: 
  void process(){
    
    ProcessedStr << "BallPos("<<X<<","<<Y<<","<<cycleno<<" cycles)"<<endl;

  }

};

/*
 * BallVel member variables: 
 * float dX - ball velocity X value 
 * float dY - ball velocity Y value
 * unsigned short cycleno - number of cycles since saw ball
 */
class MyBallVel : public BallVel {

 public: 
  void process(){
    
    ProcessedStr << "BallVel("<< dX<<","<<dY<<","<<cycleno<<" cycles)"<<endl;
  }

};


/*
 * WeHaveBall member variables: 
 * unsigned short playernum - number of player who has ball
 */
class MyWeHaveBall : public WeHaveBall {

 public: 
  void process(){

    if(playernum == 0)
      ProcessedStr << "WeHaveBall( 0 (playernum unspecified))"<<endl;
    else
      ProcessedStr << "WeHaveBall("<<playernum<<")"<<endl;
  }

};


/*
 * OppHasBall member variables: 
 * unsigned short playernum - number of player who has ball
 */
class MyOppHasBall : public OppHasBall {

 public: 
  void process(){
    if(playernum == 0)
      ProcessedStr << "OppHasBall( 0 (playernum unspecified))"<<endl;
    else
      ProcessedStr << "OppHasBall("<<playernum<<")"<<endl;
  }

};

/*
 * PassToPlayer member variables: 
 * unsigned short playernum - number of player who has ball
 */
class MyPassToPlayer : public PassToPlayer {

 public: 
  void process(){
    
    ProcessedStr << "PassToPlayer("<<playernum<<")"<<endl;
  }

};

/*
 * PassToCoord member variables: 
 * float X - X coordinate of position we're passing to.
 * float Y - Y coordinate of position we're passing to. 
 */
class MyPassToCoord : public PassToCoord {

 public: 
  void process(){
    
    ProcessedStr << "PassToCoord("<<X<<","<<Y<<")"<<endl;
  }

};


/*
 * WantPass member variables:
 * none
 */
class MyWantPass : public WantPass {

 public: 
  void process(){
    
    ProcessedStr << "WantPass()"<<endl;

  }

};


/*
 * OppPos member variables: 
 * float X - opponent position X value 
 * float Y - opponent position Y value
 * unsigned short cycleno - number of cycles since saw player
 * unsigned short playernum - # of player at pos X,Y
 */
class MyOppPos : public OppPos {

 public: 
  void process(){
    
    ProcessedStr << "OppPos(pnum "<<playernum<<","<<X<<","<<Y<<","<<cycleno<<" cycles)"<<endl;
  }

};



/*
 * TeammatePos member variables: 
 * float X - teammate position X value 
 * float Y - teammate position Y value
 * unsigned short cycleno - number of cycles since saw player
 * unsigned short playernum - # of player at pos X,Y
 */
class MyTeammatePos : public TeammatePos {

 public: 
  void process(){
    
    ProcessedStr << "TeammatePos(pnum "<<playernum<<","<<X<<","<<Y<<","<<cycleno<<" cycles)"<<endl;
  }

};



#endif
