/*
    Coachable Agent communication encoding and decoding library
    Copyright (C) 2004, John Davin, Carnegie Mellon University

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

/*
 * main.C - Simple utility program that you invoke with an encoded 
 *  say message string on the command line and it prints (to stdout)
 *  a human-readable representation of the message. 
 *  (you can pipe or redirect the output to use it however you wish - 
 *   eg, for logging to a file). 
 *
 * Creation Date: 12/3/03
 * Author: John Davin
 *
 *
 *
 */


#include "SayMsgEncoder.h"
#include "SayMsgDecoder.h"
#include "Logging_SayMsgFactory.h"

void printUsage();

// global string that the Logging_SayMsgTypes will use to store the text string
// that the msg type translates to (in process()). 
// The main loop will retrieve this string from this variable. 
stringstream ProcessedStr;


/*
 * usage: ./a.out <encoded msg string>
 *
 */
int main(int argc, char *argv[]){
  
  if(argc != 2)
    printUsage();


  string theSayMsg(argv[1]);
  SayMsgFactory *msgfactory = new Logging_SayMsgFactory();
  SayMsgDecoder *decoder = new SayMsgDecoder(msgfactory);
  bool success = decoder->decodeStr(theSayMsg, 2); //playernum doesn't matter
  if(!success){ 
    cout << "Failed to decode message: "<<theSayMsg<<endl;
    return 1;
  }
  SayMsgDecoder::DecodedMsgIterator iter = decoder->getMsgIterator();
  for( ; iter != decoder->getMsgIteratorEnd(); iter++){
    SayMsgTypes *curunit = *iter;
    curunit->process();
    cout << ProcessedStr.rdbuf();
  }


  return 0;
}



void printUsage(){
  cout << "Usage: ./a.out <encoded msg string>"<<endl;
  exit(1);

}

